package com.fr.plugin.decision.core;

import com.fr.cache.Attachment;
import com.fr.cache.list.IntList;
import com.fr.data.core.db.DBUtils;
import com.fr.data.core.db.dialect.Dialect;
import com.fr.data.core.db.dialect.base.DialectKeyConstants;
import com.fr.data.core.db.dialect.base.key.init.table.DialectInitTableParameter;
import com.fr.data.core.db.dml.CreateTable;
import com.fr.data.core.db.dml.Delete;
import com.fr.data.core.db.dml.Table;
import com.fr.data.core.db.tableObject.Column;
import com.fr.data.core.db.tableObject.ColumnSize;
import com.fr.data.dao.*;
import com.fr.file.DatasourceManager;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.utils.JdbcUtils;
import com.fr.stable.StableUtils;
import com.fr.web.AttachmentHelper;
import com.fr.workspace.WorkContext;

import java.io.*;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.*;

public class DML {

    public static void initTable(Dialect dialect, Connection conn, Table table, String connectionName, List<String> tableColName, List<String> tableColLength, List<Integer> tableColType) throws Exception {
        FieldColumnMapper[] fieldColumnMappers = new FieldColumnMapper[tableColName.size()];
        for (int k=0; k<tableColName.size(); k++){
            String[] colLen  = tableColLength.get(k).split(",");
            ColumnSize columnSize;
            if (colLen.length == 1) {
                columnSize = new ColumnSize(Integer.parseInt(colLen[0]));
            } else {
                columnSize = new ColumnSize(Integer.parseInt(colLen[0]), Integer.parseInt(colLen[1]));
            }
            CommonFieldColumnMapper fieldColumnMapper = new CommonFieldColumnMapper(
                    tableColName.get(k),
                    tableColType.get(k),
                    columnSize);
            fieldColumnMappers[k] = fieldColumnMapper;
        }
        Class clazz = conn.getClass();
        ObjectTableMapper objectTableMapper = new ObjectTableMapper(clazz, table, fieldColumnMappers);
        // 不校验 table 是否存在
        objectTableMapper.applyNoCheckTableObjectCase();
        com.fr.data.impl.Connection implconn =  DatasourceManager.getInstance().getConnection(connectionName);
        ObjectMappingTable objectMappingTable = ObjectMappingTable.createObjectMappingTable(objectTableMapper, new HashMap());
        conn.setAutoCommit(false);
        dialect.execute(DialectKeyConstants.INIT_TABLE_KEY, new DialectInitTableParameter(
                new JDBCDataAccessObjectOperator(implconn, new ObjectTableMapper[]{}),
                conn,
                new ObjectMappingTable[]{objectMappingTable}));
    }

    public static void createTable(Dialect dialect, Connection conn, Table table, List<String> tableColName, List<String> tableColLength, List<Integer> tableColType, List<Boolean> tablePrimaryKey) throws SQLException {
        CreateTable createTable = new CreateTable(table, dialect);
        for (int i=0; i<tableColName.size(); i++) {
            Column column = new Column();
            column.setName(tableColName.get(i));
            column.setValueType(tableColType.get(i));
            String[] colLen  = tableColLength.get(i).split(",");
            if (colLen.length > 1) {
                column.setColumnSize(new ColumnSize(Integer.parseInt(colLen[0]), Integer.parseInt(colLen[1])));
            } else {
                column.setColumnSize(new ColumnSize(Integer.parseInt(tableColLength.get(i))));
            }
            if (tablePrimaryKey.get(i)) {
                column.setPrimaryKey(true);
                column.setAllowNull(false);
            }
            createTable.addColumn(column);
        }
        PreparedStatement createPs = createTable.createPreparedStatementByValidatedParameters(conn);
        createPs.execute();
    }

    public static int deleteTable(Dialect dialect, Connection conn, Table table) throws SQLException {
        Delete delete = new Delete(table, dialect);
        PreparedStatement deletePs = delete.createPreparedStatementByValidatedParameters(conn);
        return deletePs.executeUpdate();
    }

    public static void dropTable(Dialect dialect, Connection conn, Table table) throws SQLException {
        Drop drop = new Drop(table, dialect);
        PreparedStatement dropPs = drop.createPreparedStatementByValidatedParameters(conn);
        dropPs.execute();
    }

    public static PreparedStatement createInsertedPreparedStatement(Dialect dialect, Connection conn, Table table, List<String> tableColName) throws SQLException {
        IntList intList = new IntList();
        ArrayList list = new ArrayList();
        ArrayList list1 = new ArrayList();
        int i = 0;

        for(int colSize = tableColName.size(); i < colSize; ++i) {
            list.add(dialect.column2SQL(tableColName.get(i)));
            list1.add("?");
            intList.add(i);
        }

        if (list1.isEmpty()) {
            return null;
        } else {
            String sql = "INSERT INTO " + table.toStatementSQLString(dialect) + " (" + StableUtils.join(list, ",") + ") VALUES (" + StableUtils.join(list1, ",") + ")";
            FineLoggerFactory.getLogger().sql("SQL: " + sql);
            return conn.prepareStatement(sql);
        }
    }

    public static int execInsertedPreparedStatement(Dialect dialect, Connection conn, Table table,PreparedStatement ps, int rowCount, List<String> tableColName, List<Integer> tableColType, Map<String, List> tableValue) throws SQLException, IOException {
        int count = 0;
        boolean supportsBatchUpdates = conn.getMetaData().supportsBatchUpdates();
        if (JdbcUtils.isHive(conn)) {
            return execHiveInsert(dialect, conn, table, ps, rowCount, tableColName, tableColType, tableValue);
        }

        if (!supportsBatchUpdates) {
            return execInsertedPreparedStatementByItem(dialect, conn, table, ps, rowCount, tableColName, tableColType, tableValue);
        }

        for (int rownum=0; rownum<rowCount; rownum++) {
            for (int index=0; index<tableColName.size(); index++) {
                int type = tableColType.get(index);
                Object value = tableValue.get(tableColName.get(index)).get(rownum);
                // excel 中读数 1 会识别为 1.0 , 列格式为[文本]时需要调整下
                if(type == 12 && JdbcUtils.isNumeric(value.toString())) {
                    value = value.toString().split("\\.")[0];
                }

                DBUtils.applyColumnTypeValue(
                        dialect,
                        conn,
                        table,
                        null,
                        null,
                        ps,
                        index + 1,
                        type,
                        value
                );
            }
            count ++ ;
            ps.addBatch();
            if(rownum!=0 && rownum%10000==0){
                ps.executeBatch();
                ps.clearBatch();
            }
        }
        ps.executeBatch();
        ps.clearBatch();
        return count;
    }

    public static int execInsertedPreparedStatementByItem(Dialect dialect, Connection conn, Table table,PreparedStatement ps, int rowCount, List<String> tableColName, List<Integer> tableColType, Map<String, List> tableValue) throws SQLException {
        int count = 0;

        for (int rownum=0; rownum<rowCount; rownum++) {
            for (int index=0; index<tableColName.size(); index++) {
                int type = tableColType.get(index);
                Object value = tableValue.get(tableColName.get(index)).get(rownum);
                // excel 中读数 1 会识别为 1.0 , 列格式为[文本]时需要调整下
                if(type == 12 && JdbcUtils.isNumeric(value.toString())) {
                    value = value.toString().split("\\.")[0];
                }

                DBUtils.applyColumnTypeValue(
                        dialect,
                        conn,
                        table,
                        null,
                        null,
                        ps,
                        index + 1,
                        type,
                        value
                );
            }
            count ++ ;
            ps.execute();
        }
        return count;
    }

    public static int execHiveInsert(Dialect dialect, Connection conn, Table table,PreparedStatement ps, int rowCount, List<String> tableColName, List<Integer> tableColType, Map<String, List> tableValue){
        try {
            String txt = hiveInsertCreateTxt(tableValue);
            DBUtils.close(ps);
            String sql = "LOAD DATA LOCAL INPATH '" + txt +"' OVERWRITE into table " + dialect.table2SQL(table);
            FineLoggerFactory.getLogger().info("execHiveInsert insert sql:" + sql);
            PreparedStatement hiveInsertPs = conn.prepareStatement(sql);
            hiveInsertPs.execute();
            FineLoggerFactory.getLogger().info("execHiveInsert:执行正常");
        } catch (SQLException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }

        return rowCount;
    }

    public static String hiveInsertCreateTxt(Map<String, List> tableValue) throws IOException {
        StringBuffer txtStr = new StringBuffer();
        String path = new StringBuilder().append(WorkContext.getCurrent().getPath()).append("/txt/hw_excel_hive").append((int)(Math.random() * 10000)).append(".txt").toString() ;
        FineLoggerFactory.getLogger().info("hiveInsert_path: " + path);

        File writename = new File(path);
        if(!writename.exists()){
            writename.createNewFile();
        }

        BufferedWriter out = new BufferedWriter(new FileWriter(writename));

        Set set = tableValue.entrySet();
        Iterator iterator = set.iterator();
        while (iterator.hasNext()){
            Map.Entry entry = (Map.Entry)iterator.next();
            List<String> colItems = (ArrayList<String>) entry.getValue();
            for (String item : colItems) {
                txtStr.append(item).append("\001");
            }
            txtStr.deleteCharAt(txtStr.length()-4);
            txtStr.append("\n");
        }
        txtStr.deleteCharAt(txtStr.length()-2);
        out.write(txtStr.toString());
        out.flush();
        out.close();

        FineLoggerFactory.getLogger().info("hiveInsert_path: " + path + " 文件创建成功");
        return path;
    }
}

