package com.fr.plugin.decision.dao;

import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.ExcelMngConfigAccessBridge;
import com.fr.stable.StringUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.condition.impl.QueryConditionImpl;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.web.utils.WebUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class ExcelLinkService {

    private static ExcelLinkService instance ;

    private ExcelLinkService(){
    }

    public static synchronized ExcelLinkService getInstance() {
        if (instance == null) {
            instance = new ExcelLinkService();
        }
        return instance;
    }

    public void addLink(final String name, final String datasouces, final String schema, final String tablename, final String createUser){
        try {
            ExcelMngConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    ExcelMngEntity entity=new ExcelMngEntity();

                    entity.setId(UUID.randomUUID().toString());
                    entity.setName(name);
                    entity.setDatasouces(datasouces);
                    entity.setSchema(schema);
                    entity.setTablename(tablename);
                    entity.setCreateUser(createUser);

                    daoContext.getDAO(ExcelMngDao.class).add(entity);
                    return entity;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接添加失败");
            e.printStackTrace();
        }
    }

    public void deleteLinkById(final String id){
        try {
            ExcelMngConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(ExcelMngDao.class).remove(id);
                    return null;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接删除失败");
            e.printStackTrace();
        }
    }

    public void deleteLinkByName(final String name){
        try {
            ExcelMngConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(ExcelMngDao.class).remove(new QueryConditionImpl().addRestriction(RestrictionFactory.eq("name", name)));
                    return null;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接删除失败");
            e.printStackTrace();
        }
    }

    public ExcelMngEntity getLinkById(final String id){
        ExcelMngEntity entity = null;
        try {
            entity = ExcelMngConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ExcelMngDao.class).findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("id",id)));
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接查询失败");
            e.printStackTrace();
        }
        return entity;
    }

    public ExcelMngEntity getLinkByName(final String name){
        ExcelMngEntity entity = null;
        try {
            entity = ExcelMngConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ExcelMngDao.class).findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("name",name)));
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接查询失败");
            e.printStackTrace();
        }
        return entity;
    }

    public List<ExcelMngEntity> getAllLink(final String key, final String createUser){
        List<ExcelMngEntity> entities = new ArrayList<>();
        final QueryCondition queryCondition = StringUtils.isBlank(key)? QueryFactory.create().addSort("name"):QueryFactory.create().addSort("name").addRestriction(RestrictionFactory.like("name", key));

        try {
            if (!UserService.getInstance().isAdmin(createUser)){
                queryCondition.addRestriction(RestrictionFactory.eq("createUser", createUser));
            }

            entities = ExcelMngConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<ExcelMngEntity> >() {
                @Override
                public List<ExcelMngEntity> run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ExcelMngDao.class).find(queryCondition);
                }
            });
        } catch (Exception e){
            FineLoggerFactory.getLogger().error("HW_excel导入:查询全部链接异常");
            e.printStackTrace();
        }
        return entities;
    }

}
