package com.fr.plugin.decision.dao;

import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.ExcelMngConfigAccessBridge;
import com.fr.plugin.decision.ReuseConfigAccessBridge;
import com.fr.stable.StringUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.condition.impl.QueryConditionImpl;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.*;

public class ReuseService {
    private static ReuseService instance ;

    private ReuseService(){
    }

    public static synchronized ReuseService getInstance() {
        if (instance == null) {
            instance = new ReuseService();
        }
        return instance;
    }

    public void addUserOnLinkName(final String userNames, final String linkName){
        try {
            ReuseConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<ReuseEntity>() {
                @Override
                public ReuseEntity run(DAOContext daoContext) throws Exception {
                    //for (String user:userNames){
                        ReuseEntity entity = new ReuseEntity();
                        entity.setId(UUID.randomUUID().toString());
                        entity.setLinkName(linkName);
                        entity.setUserName(userNames);
                        daoContext.getDAO(ReuseDao.class).add(entity);
                    //}
                    return null;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:复用添加用户失败");
            e.printStackTrace();
        }

    }

    public void deleteUserOnLinkName(final String userNames, final String linkName){
        try {
            ReuseConfigAccessBridge.getDbAccessor().runDMLAction(new DBAction<ReuseEntity>() {
                @Override
                public ReuseEntity run(DAOContext daoContext) throws Exception {
                    daoContext.getDAO(ReuseDao.class).remove(new QueryConditionImpl().addRestriction(RestrictionFactory.eq("linkName", linkName)).addRestriction(RestrictionFactory.eq("userName",userNames)));
                    return null;
                }
            });
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("HW_excel导入:链接删除失败");
            e.printStackTrace();
        }

    }

    public List<ReuseEntity> getUserByLinkName(final String linkName){
        List<ReuseEntity> entities = new ArrayList<>();
        final QueryCondition queryCondition = QueryFactory.create().addSort("userName").addRestriction(RestrictionFactory.eq("linkName", linkName));

        try {
            entities = ReuseConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<ReuseEntity> >() {
                @Override
                public List<ReuseEntity> run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ReuseDao.class).find(queryCondition);
                }
            });
        } catch (Exception e){
            FineLoggerFactory.getLogger().error("HW_excel导入:getUserByLinkName异常");
            e.printStackTrace();
        }
        return entities;
    }

    public List<ReuseEntity> getLinkByUserName(final String userName){
        List<ReuseEntity> entities = new ArrayList<>();
        final QueryCondition queryCondition = QueryFactory.create().addSort("linkName").addRestriction(RestrictionFactory.eq("userName", userName));

        try {
            entities = ReuseConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<List<ReuseEntity> >() {
                @Override
                public List<ReuseEntity> run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ReuseDao.class).find(queryCondition);
                }
            });
        } catch (Exception e){
            FineLoggerFactory.getLogger().error("HW_excel导入:getLinkByUserName异常");
            e.printStackTrace();
        }
        return entities;
    }

}
