package com.fr.plugin.decision.http;

import com.fr.data.core.db.ColumnInformation;
import com.fr.data.core.db.DBUtils;
import com.fr.data.core.db.dialect.Dialect;
import com.fr.data.core.db.dialect.DialectFactory;
import com.fr.data.core.db.dml.Table;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.plugin.decision.ExcelMngConfigAccessBridge;
import com.fr.plugin.decision.dao.ExcelMngDao;
import com.fr.plugin.decision.dao.ExcelMngEntity;
import com.fr.plugin.decision.request.handler.HWAbstractHandler;
import com.fr.plugin.decision.utils.JdbcUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

public class QueryTableDataHttpHandle extends HWAbstractHandler {

    private int queryCount = 5000;
    private JSONObject para ;
    private JSONObject result ;

    @Override
    public RequestMethod getMethod() {
        return null;
    }

    @Override
    public String getPath() {
        return "/excelmng/querydata";
    }

    @Override
    public boolean isPublic() {
        return false;
    }

    public int getQueryCount() {
        return queryCount;
    }

    public void setQueryCount(int queryCount) {
        this.queryCount = queryCount;
    }

    @Override
    protected void deal(HttpServletRequest req, HttpServletResponse res, JSONObject para, JSONObject result) throws Exception {
        final String name = para.getString("name");
        ExcelMngEntity entity = null;
        try {
            entity = ExcelMngConfigAccessBridge.getDbAccessor().runQueryAction(new DBAction<ExcelMngEntity>() {
                @Override
                public ExcelMngEntity run(DAOContext daoContext) throws Exception {
                    return daoContext.getDAO(ExcelMngDao.class).findOne(QueryFactory.create().addRestriction(RestrictionFactory.eq("name",name)));
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
            WebUtils.printAsJSON(res,new JSONObject().put("status","fail"));
        }

        String connection = entity.getDatasouces();
        String tableName  = entity.getTablename();
        String schema = entity.getSchema();
        Connection conn = null;
        Statement smt = null;
        List<String> primaryKeyCol = new ArrayList<>();

        try {
            conn = JdbcUtils.getConnection(connection);
            smt = conn.createStatement();
            Dialect dialect = DialectFactory.generateDialect(conn);
            Table table = new Table(schema, tableName);
            ResultSet resultSet = smt.executeQuery(dialect.getTopNRowSql(queryCount, table));
            ColumnInformation[] columnInformations = dialect.getColumnInformation(conn, resultSet, StringUtils.EMPTY, StringUtils.EMPTY, StringUtils.EMPTY);

            /*
             * catalog - 表所在的类别名称;""表示获取没有类别的列,null表示获取所有类别的列。
			 * schema - 表所在的模式名称(oracle中对应于Tablespace);""表示获取没有模式的列,null标识获取所有模式的列; 可包含单字符通配符("_"),或多字符通配符("%");
			 * table - 表名称;可包含单字符通配符("_"),或多字符通配符("%");
			 */
            DatabaseMetaData databaseMetaData = conn.getMetaData();
            ResultSet primaryKeysResult = databaseMetaData.getPrimaryKeys(null, schema, tableName);
            while(primaryKeysResult.next()){
                primaryKeyCol.add(primaryKeysResult.getString(4));
            }
            JSONArray tableData = JdbcUtils.formatRsToTableData(resultSet);
            JSONArray colAttr = JdbcUtils.formatRs2ColAttr(columnInformations);
            JdbcUtils.addPrimaryKeyColAttr(colAttr, primaryKeyCol);

            result.put(SUCCESS,"success").put("data",
                    new JSONObject()
                            .put("tableData", tableData)
                            .put("colAttr", colAttr)
                            .put("tableAttr", new JSONObject()
                                    .put("connection", connection)
                                    .put("schema", schema)
                                    .put("tableName", tableName)
                                    .put("aliasName", name)
                            )
            );
        } catch (Exception e){
            e.printStackTrace();
        } finally {
            DBUtils.close(smt);
            DBUtils.close(conn);
        }
    }
}
