!(function () {

    var WIDTH = 80;
    var HEIGHT = 50;

    var AddDepartment = BI.inherit(BI.Widget, {

        props: {
            baseCls: "dec-add-department-popup",
            btns: [BI.i18nText(BI.i18nText("Dec-Basic_Sure")), BI.i18nText(BI.i18nText("Dec-Basic_Cancel"))],
            onClickConfirm: BI.emptyFn,
            info: {}
        },

        _store: function () {
            return BI.Models.getModel("dec.model.hw.table.add.popup");
        },

        watch:{
            importStatus: function (v) {
                switch (v) {
                    case "importing":
                        this._startImport();
                        break;
                    case "abort":
                        this._stopImport();
                        break;
                    case "finish":
                    default:
                        this._finishImport();
                }
            }
        },

        render: function () {
            var self = this, o = this.options;

            return {
                type: "bi.vtape",
                items: [
                    {
                        el: {
                            type: "dec.hw.table.info",
                            add: o.add,
                            infoId:o.infoId,
                            aliasName: o.aliasName,
                            ref: function (_ref) {
                                self.hwTableInfo = _ref;
                            }
                        }
                    }, {
                        el: this.rebuildSouth(),
                        height: 44
                    }
                ]
            };
        },


        rebuildSouth: function () {
            var self = this, o = this.options;
            var sure = BI.createWidget({
                type: "bi.button",
                ref: function (_ref) {
                    self.sure = _ref;
                },
                text: this.options.btns[0],
                warningTitle: o.warningTitle,
                height: 24,
                handler: function () {
                    self.end();
                }
            });
            var cancel = BI.createWidget({
                type: "bi.button",
                ref: function (_ref) {
                    self.cancel = _ref;
                },
                text: this.options.btns[1],
                height: 24,
                level: "ignore",
                handler: function () {
                    self.close();
                }
            });
            return {
                type: "bi.right_vertical_adapt",
                lgap: 10,
                items: [cancel, sure]
            };
        },

        getValue: function () {
            return this.hwTableInfo.getValue();
        },

        setValue: function (info) {
            this.departmentName.setValue(info.text);
        },

        close: function () {
            this.fireEvent("EVENT_CLOSE");
        },

        end: function () {
            var self = this;
            self.store.setImportStatus("importing");

            if (self.getValue().infoId == "add") {
                self._tableNameExistsAndImportChecker();
            } else {
                self._importChecker();
            }
        },

        _tableNameExistsAndImportChecker: function(){
            var self = this;
            if (self.getValue().infoId == "add") {
                var tableName = BI.isEmpty(self.getValue().schema)?self.getValue().tableName:self.getValue().schema+ "." +　self.getValue().tableName ;
                var encodeTableName = encodeURIComponent(tableName); // 编码下表名，模式中可能存在 #
                Dec.reqGet(
                    "/url/excelmng/iftableexist?connection=" + self.getValue().connection + "&tableName=" + encodeTableName,
                    null,null,function(e){
                        if(e.status == "success"){
                            self._importChecker();
                        } else {
                            self.store.setImportStatus("abort");
                            BI.Msg.toast(e.errorText);
                        }
                    })
            }
        },

        _importChecker: function () {
            var self = this, o = this.options;
            o.onClickConfirm(self.getValue(), function (res) {
                if (res.status == "success") {
                    self.store.setImportStatus("finish");
                    BI.Msg.toast("操作成功！" + res.text);
                } else {
                    self.store.setImportStatus("abort");
                    BI.Msg.toast("操作失败，" + res.errorText);
                }
            })
        },

        _startImport: function () {
            var self = this, name = this.getName() + "status";
            var upload = {
                type: "dec.test.status",
                waitingText: "导入中",
                successText: "导入成功",
                failText: "导入失败",
                listeners: [{
                    eventName: "EVENT_CLOSE",
                    action: function () {
                        BI.Maskers.hide(name);
                        BI.Maskers.remove(name);
                    }
                }],
                ref: function (_ref) {
                    self.uploadMasker = _ref;
                }
            };
            BI.Maskers.create(name, null, {
                render: upload
            });
            BI.Maskers.show(name);
        },
        _stopImport: function () {
            var name = this.getName() + "status";
            BI.Maskers.hide(name);
            BI.Maskers.remove(name);
        },
        _finishImport: function () {
            this.uploadMasker && this.uploadMasker.setStatus({
                status: DecCst.TestStatus.SUCCESS
            });
            this.close();
        }
    });

    AddDepartment.EVENT_CLICK_CONFIRM = "EVENT_CONFIRM";
    BI.shortcut("dec.hw.table.add.popup", AddDepartment);
}());